#! /usr/local/bin/ruby
# encoding: utf-8

$mydir = Dir.pwd

$LOAD_PATH << "#{$mydir}/lib"
$LOAD_PATH << File.expand_path(File.dirname(__FILE__)) + "/lib"

require "net/ranidb.rb"
require "ranidb_prefs.rb"
require "ranidb_backend.rb"
require "ranidb_filemanager.rb"
require "ranidb_mylistmanager.rb"
require "ranidb_maintenance.rb"
require "ranidb_logger.rb"
require "ranidb_tools.rb"

require "logger"

log_level = nil
interrupted = false

begin

  prefs = RAniDBPreferences.instance
  prefs.load("#{$mydir}/radb.prefs")

  log_level = prefs.parse_log_level(ARGV)

  logger = RAniDBLogger.instance
  logger.start(prefs[:options][:logger],
               log_level)

  back = RAniDBBackend.instance
  back.start(prefs[:options][:database_path])
  db = back.db

  db.logger = logger.provide_logger(:sql)

  action = nil
  actions = ARGV.group_by do |a|
    if a =~ /^[a-z0-9_]+$/ && RAniDBMaintenance.respond_to?(a.to_sym)
      action = a.to_sym
    end
    action
  end

  actions.each do |action, args|
    a = args.shift
    raise "unknown action #{a}" if action.nil?
    c = RAniDBMaintenance.send(a, *args)
    puts "#{action.to_s.capitalize} : no files found." if c == 0
  end

rescue Interrupt => e
  interrupted = true
  logger.global "Interrupted"
  e.backtrace.each { |b| logger.global b }
  puts
  puts "Interrupted."
rescue RAniDBPreferencesError => e
  interrupted = true
  puts "Error parsing configuration file :"
  puts "  " + e.message
rescue Exception => e
  interrupted = true
  if logger
    logger.global "Unexpected error"
    logger.global e.message
    e.backtrace.each { |b| logger.global b }
    puts
    puts "Unexpected error : #{e.message} (#{e.class})"
  else
    puts "Unexpected error : #{e.message} (#{e.class})"
    puts
    e.backtrace.each { |b| puts b }
    puts
  end
ensure
end
logger.info "Finished." if logger
