#! /usr/local/bin/ruby
# encoding: utf-8

$mydir = File.expand_path(File.dirname(__FILE__))

$LOAD_PATH << "#{$mydir}/lib"
#$LOAD_PATH << File.expand_path(File.dirname(__FILE__)) + "/lib"

require "sequel"
require "ranidb_prefs.rb"
require "ranidb_backend.rb"
require "ranidb_logger.rb"
require "ranidb_tools.rb"

require "logger"

require "socket"

Socket.send(:remove_const, :AF_INET6) rescue nil

require "ramaze"

require "daemons"

$layout = File.read("#{$mydir}/templates/layout.xhtml")
$templates = "#{$mydir}/templates"

module Ramaze
  module Helper
    module Ranidb
      def cycle(*args)
        @_cycle ||= {}
        if args[0].is_a? Symbol
          t = args.shift
        else
          t = :default
        end
        @_cycle[t] ||= -1
        @_cycle[t] = (@_cycle[t] + 1) % args.length
        args[@_cycle[t]]
      end

      def reset_cycle(type = :default)
        @_cycle ||= {}
        @_cycle[type] ||= -1
      end

      def format_date(d)
        if d.to_i == 0
          "Unknown"
        else
          unless d.is_a? Time
            d = Time.at(d.to_i)
          end
          d.strftime('%d/%m/%Y')
        end
      end

      def format_date_hour(d)
        if d.to_i == 0
          "Unknown"
        else
          unless d.is_a? Time
            d = Time.at(d.to_i)
          end
          d.strftime('%d/%m/%Y %H:%M:%S')
        end
      end

      def format_ep(epno)
        if epno =~ /^([a-zA-Z]+)(\d+)$/
          "%s%02d" % [ $1, $2.to_i ]
        elsif epno =~ /^(\d+)$/
          "%02d" % [ $1.to_i ]
        else
          epno
        end
      end

      def seconds_to_minutes(s)
        "%d:%02d" % [ s / 60, s % 60 ]
      end

      def listify(l)
        l.gsub(/'/, ', ')
      end

      def sizeify(s)
        case s
        when       0...1024**1 then "#{s} b"
        when 1024**1...1024**2 then "#{'%.02f' % [ s / 1024.0 ]} Kb"
        when 1024**2...1024**3 then "#{'%.02f' % [ s / 1024.0**2]} Mb"
        when 1024**3...1024**9 then "#{'%.02f' % [ s / 1024.0**3]} Gb"
        end
      end

      def anidb_href(id_type, id)
        what = case id_type
               when :aid then 'anime'
               when :gid then 'group'
               when :eid then 'ep'
               when :fid then 'file'
               end
        "<a href='http://anidb.net/perl-bin/animedb.pl?show=#{what}&#{id_type}=#{id}'>#{what} #{id}</a>"
      end

      def template(t)
        File.read("#{$templates}/#{t}.xhtml")
      end
    end
  end
end

class MainController < Ramaze::Controller
  engine :Erubis
  helper :ranidb

  def index
    template "search"
  end

  def search_files
    @files = RAniDBLocalFile
    path = request['path'] || ''
    name = request['name'] || ''
    category = request['category'] || ''
    crc32 = request['crc32'] || ''
    ed2k = request['ed2k'] || ''
    unless path.empty?
      @files = @files.where(:path.ilike("%#{path}%"))
    end
    unless name.empty?
      @files = @files.where(:name.ilike("%#{name}%"))
    end
    unless category.empty?
      @files = @files.where(:category => category)
    end
    unless ed2k.empty?
      @files = @files.where(:ed2k => ed2k)
    end
    unless (path+name+ed2k+category).empty?
      @files = @files.limit(500)
      @sql = @files.sql
      @files = @files.all
      template "search_files"
    else
      redirect "index"
    end
  end

  def search_animes
    @animes = AniDB_Anime
    name = request['name'] || ''
    year = request['year'] || ''
    unless name.empty?
      @animes = @animes.where((:romaji_title.ilike("%#{name}%")) |
                              (:english_title.ilike("%#{name}%")))
    end
    unless year.empty?
      @animes = @animes.where((:year.like("%#{year}")) |
                              (:year.like("#{year}%")))
    end
    unless (name+year).empty?
      @animes = @animes.limit(500)
      @sql = @animes.sql
      @animes = @animes.all
      if @animes.count == 1
        redirect "http://ardbeg.lacave.net/anidb/anime?aid=#{@animes.first.aid}"
      else
        template "search_animes"
      end
    else
      redirect "anidb"
    end
  end

  def anime
    aid = request['aid'] || 0
    unless aid == 0
      @anime = AniDB_Anime.find(:aid => aid)
    end
    if @anime
      template "anime"
    else
      redirect "index"
    end
  end

  def layout
    $layout.sub("{{@content}}", @content)
  end

  layout :layout
end

Daemons.run_proc('query_radb', :dir_mode => :normal, :dir => $mydir) do
  Dir.chdir($mydir) do
    $prefs = RAniDBPreferences.instance
    $prefs.load("#{$mydir}/radb.prefs")

    log_level = $prefs.parse_log_level(ARGV)

    logger = RAniDBLogger.instance
    logger.start($prefs[:options][:logger],
                 log_level)

    back = RAniDBBackend.instance
    back.start($prefs[:options][:database_path])
    $db = back.db

    $db.logger = logger.provide_logger(:sql)

    Ramaze.start :adapter => :thin, :port => $prefs[:options][:frontend][:port] #, :prefix => '/anidb'
  end
end
